/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_legend.h>
#include <qwt_text.h>
#include "DailyDataTab.h"
#include "TemperaturesGraph.h"
#include "RainGraph.h"
#include "HumidityGraph.h"
#include "PressureGraph.h"
#include "WindGraph.h"

#include "Ws2355.h"

DailyDataTab::DailyDataTab( QWidget *parent, Ws2355 *station, Configuration *config ) :
    QFrame( parent )
{
    theStation = station;
    theGlobalConfiguration = config;

    createTab();
}

/**
 * Destructor
 */
DailyDataTab::~DailyDataTab()
{
    delete daylyTemperaturesGraph;
    delete daylyRainGraph;
    delete daylyHumidityGraph;
    delete daylyPressureGraph;
    delete daylyWindGraph;
    delete dailyDataDate;

    delete temperaturesGroup;
    delete rainGroup;
    delete humidityGroup;
    delete pressureGroup;
    delete windGroup;
    delete dateGroup;
}

/**
 * When the language change, update all text fields.
 * Capture QEvent::LanguageChange and forward all other events
 *
 * @param event
 */
void DailyDataTab::changeEvent(QEvent *event) {
    if (event->type() == QEvent::LanguageChange)
    {
        // Set all texts
        temperaturesGroup->setTitle( trUtf8( "Températures" ));
        rainGroup->setTitle( trUtf8( "Précipitations" ));
        humidityGroup->setTitle( trUtf8( "Humidité" ));
        pressureGroup->setTitle( trUtf8( "Pression" ));
        windGroup->setTitle( trUtf8( "Vent" ));
        dateGroup->setTitle( trUtf8( "Date" ));

        daylyTemperaturesGraph->setText();
        daylyRainGraph->setText();
        daylyHumidityGraph->setText();
        daylyPressureGraph->setText();
        daylyWindGraph->setText();
    }
    else
    {
        QWidget::changeEvent(event);
    }
}

/**
 * Create all widgets displayed in this tab
 */
void DailyDataTab::createTab( void )
{
    // Temperatures
    temperaturesGroup = new QGroupBox( trUtf8( "Températures" ));
    daylyTemperaturesGraph =  new TemperaturesGraph( 1, 24, 1, theGlobalConfiguration->getTemperatureUnit() );

    // Rain
    rainGroup = new QGroupBox( trUtf8( "Précipitations" ));
    daylyRainGraph =  new RainGraph( 0, 24, 1 );

    // Humidity
    humidityGroup = new QGroupBox( trUtf8( "Humidité" ));
    daylyHumidityGraph =  new HumidityGraph( 1, 24, 1 );

    // Pressure
    pressureGroup = new QGroupBox( trUtf8( "Pression" ));
    daylyPressureGraph =  new PressureGraph( 1, 24, 1 );


    // Wind
    windGroup = new QGroupBox( trUtf8( "Vent" ));
    daylyWindGraph =  new WindGraph( 1, 24, 1 );

    // Date
    dateGroup = new QGroupBox( trUtf8( "Date" ));
    dailyDataDate = new QCalendarWidget;

    connect( dailyDataDate, SIGNAL( selectionChanged() ), this, SLOT( selectedDateChanged() ));

    // Set up layout
    QGridLayout *temperaturesLayout = new QGridLayout;
    temperaturesLayout->addWidget( daylyTemperaturesGraph->getPlot(), 0, 0 );
    temperaturesGroup->setLayout( temperaturesLayout );

    QGridLayout *rainLayout = new QGridLayout;
    rainLayout->addWidget( daylyRainGraph->getPlot(), 0, 0 );
    rainGroup->setLayout( rainLayout );

    QGridLayout *humidityLayout = new QGridLayout;
    humidityLayout->addWidget( daylyHumidityGraph->getPlot(), 0, 0 );
    humidityGroup->setLayout( humidityLayout );

    QGridLayout *pressureLayout = new QGridLayout;
    pressureLayout->addWidget( daylyPressureGraph->getPlot(), 0, 0 );
    pressureGroup->setLayout( pressureLayout );

    QGridLayout *windLayout = new QGridLayout;
    windLayout->addWidget( daylyWindGraph->getPlot(), 0, 0 );
    windGroup->setLayout( windLayout );

    QGridLayout *dateLayout = new QGridLayout;
    dateLayout->addWidget( dailyDataDate, 0, 0 );
    dateGroup->setLayout( dateLayout );

    QGridLayout *currentLayout = new QGridLayout();
    currentLayout->addWidget( temperaturesGroup, 0, 0 );
    currentLayout->addWidget( rainGroup, 0, 1 );
    currentLayout->addWidget( humidityGroup, 0, 2 );
    currentLayout->addWidget( pressureGroup, 1, 0 );
    currentLayout->addWidget( windGroup, 1, 1 );
    currentLayout->addWidget( dateGroup, 1, 2 );
    this->setLayout( currentLayout );

    updateGraphics( dailyDataDate->selectedDate() );
}

/**
 * SLOT : Change displayed data when a new date is selected
 */
void DailyDataTab::selectedDateChanged()
{
    updateGraphics( dailyDataDate->selectedDate() );
}

void DailyDataTab::updateGraphics( QDate date  )
{
    theStation->getTheDataLogged()->readData( date, BY_DAY );

    daylyTemperaturesGraph->update( theStation->getTheDataLogged()->getData()->inTemperatureX,
                                    theStation->getTheDataLogged()->getData()->inTemperatureY,
                                    theStation->getTheDataLogged()->getData()->outTemperatureX,
                                    theStation->getTheDataLogged()->getData()->outTemperatureY,
                                    theStation->getTheDataLogged()->getData()->dewPointTemperatureX,
                                    theStation->getTheDataLogged()->getData()->dewPointTemperatureY );

    daylyHumidityGraph->update( theStation->getTheDataLogged()->getData()->inHumidityX,
                                theStation->getTheDataLogged()->getData()->inHumidityY,
                                theStation->getTheDataLogged()->getData()->outHumidityX,
                                theStation->getTheDataLogged()->getData()->outHumidityY );

    daylyPressureGraph->update( theStation->getTheDataLogged()->getData()->pressureX,
                                theStation->getTheDataLogged()->getData()->pressureY );

    daylyRainGraph->update( theStation->getTheDataLogged()->getData()->rainX,
                            theStation->getTheDataLogged()->getData()->rainY);

    daylyWindGraph->update( theStation->getTheDataLogged()->getData()->windDirectionX,
                            theStation->getTheDataLogged()->getData()->windDirectionY,
                            theStation->getTheDataLogged()->getData()->windSpeedX,
                            theStation->getTheDataLogged()->getData()->windSpeedY );
}
