/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "MonthlyDataTab.h"

MonthlyDataTab::MonthlyDataTab( QWidget *parent, Ws2355 *station, Configuration *config ) :
    QFrame(parent)
{
    theStation = station;
    theGlobalConfiguration = config;

    createTab();
}

/**
 * Destructor
 */
MonthlyDataTab::~MonthlyDataTab()
{
    delete monthlyTemperaturesGraph;
    delete monthlyRainGraph;
    delete monthlyHumidityGraph;
    delete monthlyPressureGraph;
    delete monthlyWindGraph;
    delete month;

    delete temperaturesGroup;
    delete rainGroup;
    delete humidityGroup;
    delete pressureGroup;
    delete windGroup;
}

/**
 * When the language change, update all text fields.
 * Capture QEvent::LanguageChange and forward all other events
 *
 * @param event
 */
void MonthlyDataTab::changeEvent(QEvent *event) {
    if (event->type() == QEvent::LanguageChange)
    {
        // Set all texts
        temperaturesGroup->setTitle( trUtf8( "Températures" ));
        rainGroup->setTitle( trUtf8( "Précipitations" ));
        humidityGroup->setTitle( trUtf8( "Humidité" ));
        pressureGroup->setTitle( trUtf8( "Pression" ));
        windGroup->setTitle( trUtf8( "Vent" ));

        monthlyTemperaturesGraph->setText();
        monthlyRainGraph->setText();
        monthlyHumidityGraph->setText();
        monthlyPressureGraph->setText();
        monthlyWindGraph->setText();
    }
    else
    {
        QWidget::changeEvent(event);
    }
}

void MonthlyDataTab::createTab( void )
{
    // Temperatures
    temperaturesGroup = new QGroupBox( trUtf8( "Températures" ));

    monthlyTemperaturesGraph =  new TemperaturesGraph( 1, 31, 1, theGlobalConfiguration->getTemperatureUnit());

    // Rain
    rainGroup = new QGroupBox( trUtf8( "Précipitations" ));

    monthlyRainGraph =  new RainGraph( 0, 31, 1);

    // Humidity
    humidityGroup = new QGroupBox( trUtf8( "Humidité" ));

    monthlyHumidityGraph =  new HumidityGraph( 1, 31, 1);

    // Pressure
    pressureGroup = new QGroupBox( trUtf8( "Pression" ));

    monthlyPressureGraph =  new PressureGraph( 1, 31, 1);

    // Wind
    windGroup = new QGroupBox( trUtf8( "Vent" ));

    monthlyWindGraph =  new WindGraph( 1, 31, 1);

    // Month
    QGroupBox *monthGroup = new QGroupBox( );

    month = new QDateEdit( );
    month->setDisplayFormat( "MMMM yyyy" );
    month->setMaximumSize( 200,35 );
    month->setDate( QDate::currentDate() );
    connect( month, SIGNAL( dateChanged( QDate ) ), this, SLOT( monthChanged( QDate ) ));

    // Set up layout

    QGridLayout *temperaturesLayout = new QGridLayout;
    temperaturesLayout->addWidget( monthlyTemperaturesGraph->getPlot(), 0, 0 );
    temperaturesGroup->setLayout( temperaturesLayout );

    QGridLayout *rainLayout = new QGridLayout;
    rainLayout->addWidget( monthlyRainGraph->getPlot(), 0, 0 );
    rainGroup->setLayout( rainLayout );

    QGridLayout *humidityLayout = new QGridLayout;
    humidityLayout->addWidget( monthlyHumidityGraph->getPlot(), 0, 0 );
    humidityGroup->setLayout( humidityLayout );

    QGridLayout *pressureLayout = new QGridLayout;
    pressureLayout->addWidget( monthlyPressureGraph->getPlot(), 0, 0 );
    pressureGroup->setLayout( pressureLayout );

    QGridLayout *windLayout = new QGridLayout;
    windLayout->addWidget( monthlyWindGraph->getPlot(), 0, 0 );
    windGroup->setLayout( windLayout );

    QGridLayout *monthLayout = new QGridLayout;
    monthLayout->addWidget( month, 0, 0 );
    monthGroup->setLayout( monthLayout );

    QGridLayout *currentLayout = new QGridLayout();
    currentLayout->addWidget( temperaturesGroup, 0, 0 );
    currentLayout->addWidget( rainGroup, 0, 1 );
    currentLayout->addWidget( humidityGroup, 0, 2 );
    currentLayout->addWidget( pressureGroup, 1, 0 );
    currentLayout->addWidget( windGroup, 1, 1 );
    currentLayout->addWidget( monthGroup, 1, 2 );

    this->setLayout( currentLayout );

    updateGraphics( month->date() );
}

/**
 * SLOT : Change displayed data when a new month is selected
 */
void MonthlyDataTab::monthChanged( const QDate & date )
{
    monthlyTemperaturesGraph->setXAxisMax( date.daysInMonth() );
    monthlyRainGraph->setXAxisMax( date.daysInMonth() );
    monthlyHumidityGraph->setXAxisMax( date.daysInMonth() );
    monthlyPressureGraph->setXAxisMax( date.daysInMonth() );
    monthlyWindGraph->setXAxisMax( date.daysInMonth() );

    updateGraphics( month->date() );

}

void MonthlyDataTab::updateGraphics( QDate date  )
{
    theStation->getTheDataLogged()->readData( date, BY_MONTH );

    monthlyTemperaturesGraph->update( theStation->getTheDataLogged()->getData()->inTemperatureX,
                                      theStation->getTheDataLogged()->getData()->inTemperatureY,
                                      theStation->getTheDataLogged()->getData()->outTemperatureX,
                                      theStation->getTheDataLogged()->getData()->outTemperatureY,
                                      theStation->getTheDataLogged()->getData()->dewPointTemperatureX,
                                      theStation->getTheDataLogged()->getData()->dewPointTemperatureY );

    monthlyHumidityGraph->update( theStation->getTheDataLogged()->getData()->inHumidityX,
                                  theStation->getTheDataLogged()->getData()->inHumidityY,
                                  theStation->getTheDataLogged()->getData()->outHumidityX,
                                  theStation->getTheDataLogged()->getData()->outHumidityY );

    monthlyPressureGraph->update( theStation->getTheDataLogged()->getData()->pressureX,
                                  theStation->getTheDataLogged()->getData()->pressureY );

    monthlyRainGraph->update( theStation->getTheDataLogged()->getData()->rainX,
                              theStation->getTheDataLogged()->getData()->rainY);

    monthlyWindGraph->update( theStation->getTheDataLogged()->getData()->windDirectionX,
                              theStation->getTheDataLogged()->getData()->windDirectionY,
                              theStation->getTheDataLogged()->getData()->windSpeedX,
                              theStation->getTheDataLogged()->getData()->windSpeedY );
}

