/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *  Based on findings and code published                                   *
 *  by Kenneth Lavrsen, see http://www.lavrsen.dk                          *
 ***************************************************************************/
#ifndef WS2355_H
#define WS2355_H

#include <QString>

#include "CurrentData.h"
#include "Connection.h"
#include "Configuration.h"
#include "WsMeteo.h"
#include "DataLogged.h"


class CurrentData;
class Ws2355Thread;

#define FORECAST_MAX_NAME_LENGTH    7
#define TENDENCY_MAX_NAME_LENGTH    8

#define MAX_NUMBER_OF_RECORDS       175

#define MAXRETRIES          50
#define MAXWINDRETRIES      20
#define WRITENIB            0x42
#define SETBIT              0x12
#define UNSETBIT            0x32
#define WRITEACK            0x10
#define SETACK              0x04
#define UNSETACK            0x0C
#define RESET_MIN           0x01
#define RESET_MAX           0x02

#define METERS_PER_SECOND   1.0
#define KILOMETERS_PER_HOUR 3.6
#define MILES_PER_HOUR      2.23693629

#define CELCIUS             0
#define FAHRENHEIT          1

#define MILLIMETERS         1
#define INCHES              25.4

#define HECTOPASCAL         1.0
#define MILLIBARS           1.0
#define INCHES_HG           33.8638864


// BITs in alarm registers
#define PRESSURE_MAX_ALARM_BIT              0x8
#define PRESSURE_MIN_ALARM_BIT              0x4
#define OUTDOOR_TEMPERATURE_MAX_ALARM_BIT   0x8
#define OUTDOOR_TEMPERATURE_MIN_ALARM_BIT   0x4
#define INDOOR_TEMPERATURE_MAX_ALARM_BIT    0x2
#define INDOOR_TEMPERATURE_MIN_ALARM_BIT    0x1
#define DEWPOINT_MAX_ALARM_BIT              0x3
#define DEWPOINT_MIN_ALARM_BIT              0x2
#define WINDCHILL_MAX_ALARM_BIT             0x1
#define WINDCHILL_MIN_ALARM_BIT             0x0
#define OUTDOOR_HUMIDITY_MAX_ALARM_BIT      0x8
#define OUTDOOR_HUMIDITY_MIN_ALARM_BIT      0x4
#define INDOOR_HUMIDITY_MAX_ALARM_BIT       0x2
#define INDOOR_HUMIDITY_MIN_ALARM_BIT       0x1
#define RAIN_1H_ALARM_BIT                   0x2
#define RAIN_24H_ALARM_BIT                  0x1
#define WIND_DIR_ALARM_BIT                  0x4
#define WIND_SPEED_MAX_ALARM_BIT            0x2
#define WIND_SPEED_MIN_ALARM_BIT            0x1

/**
 * Memory mapping of a WS23XX weather station.
 *
 * These values are derived from
 * http://www.lavrsen.dk/foswiki/bin/view/Open2300/OpenWSMemoryMap
 */
#define PRESSURE_ALARM                      0x01A
#define INDOOR_TEMPERATURE_ALARM            0x01B
#define OUTDOOR_TEMPERATURE_ALARM           0x01B
#define DEWPOINT_ALARM                      0x01C
#define WINDCHILL_ALARM                     0x01C
#define INDOOR_HUMIDITY_ALARM               0x01D
#define OUTDOOR_HUMIDITY_ALARM              0x01D
#define RAIN_ALARM                          0x01E
#define WIND_DIRECTION_ALARM                0x01F
#define WIND_SPEED_ALARM                    0x01F

#define UTC_CURRENT_TIME                    0x200        // BCD 1s, 10s, 1mn, 10mn, 1h, 10h
#define UTC_CURRENT_WEEKDAY                 0x206
#define UTC_CURRENT_DAY_OF_MONTH            0x207        // BCD 1s, 10s
#define UTC_CURRENT_MONTH                   0x209        // BCD 1s, 10s
#define UTC_CURRENT_YEAR                    0x20B        // BCD 1s, 10s
#define LOCAL_CURRENT_TIME                  0x239        // BCD 1s, 10s, 1mn, 10mn, 1h, 10h
#define LOCAL_CURRENT_WEEKDAY               0x23F
#define LOCAL_CURRENT_DAY_OF_MONTH          0x240        // BCD 1s, 10s
#define LOCAL_CURRENT_MONTH                 0x242        // BCD 1s, 10s
#define LOCAL_CURRENT_YEAR                  0x244        // BCD 1s, 10s

#define FORECAST                            0x26B
#define TENDENCY                            0x26C

#define CURRENT_INDOOR_TEMPERATURE          0x346        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define MINIMUM_INDOOR_TEMPERATURE          0x34B        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define MAXIMUM_INDOOR_TEMPERATURE          0x350        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define TIME_MINIMUM_INDOOR_TEMPERATURE     0x354        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MINIMUM_INDOOR_TEMPERATURE     0x358        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define TIME_MAXIMUM_INDOOR_TEMPERATURE     0x35E        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_INDOOR_TEMPERATURE     0x362        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define LOW_ALARM_INDOOR_TEMPERATURE        0x369        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define HIGH_ALARM_INDOOR_TEMPERATURE       0x36E        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define CURRENT_OUTDOOR_TEMPERATURE         0x373        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define MINIMUM_OUTDOOR_TEMPERATURE         0x378        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define MAXIMUM_OUTDOOR_TEMPERATURE         0x37D        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define TIME_MINIMUM_OUTDOOR_TEMPERATURE    0x381        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MINIMUM_OUTDOOR_TEMPERATURE    0x385        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define TIME_MAXIMUM_OUTDOOR_TEMPERATURE    0x38B        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_OUTDOOR_TEMPERATURE    0x38F        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define LOW_ALARM_OUTDOOR_TEMPERATURE       0x396        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define HIGH_ALARM_OUTDOOR_TEMPERATURE      0x39B        // BCD offset 30 0.01s, 0.1s, 1s, 10s

#define CURRENT_WINDCHILL                   0x3A0        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define MINIMUM_WINDCHILL                   0x3A5        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define MAXIMUM_WINDCHILL                   0x3AA        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define TIME_MINIMUM_WINDCHILL              0x3AE        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MINIMUM_WINDCHILL              0x3B2        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define TIME_MAXIMUM_WINDCHILL              0x3B8        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_WINDCHILL              0x3BC        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define LOW_ALARM_WINDCHILL                 0x3C3        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define HIGH_ALARM_WINDCHILL                0x3C8        // BCD offset 30 0.01s, 0.1s, 1s, 10s

#define CURRENT_DEWPOINT                    0x3CE        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define MINIMUM_DEWPOINT                    0x3D3        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define MAXIMUM_DEWPOINT                    0x3D8        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define TIME_MINIMUM_DEWPOINT               0x3DC        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MINIMUM_DEWPOINT               0x3DE        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define TIME_MAXIMUM_DEWPOINT               0x3E6        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_DEWPOINT               0x3EA        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define LOW_ALARM_DEWPOINT                  0x3F1        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define HIGH_ALARM_DEWPOINT                 0x3F6        // BCD offset 30 0.01s, 0.1s, 1s, 10s

#define REL_HUMIDITY_INDOOR                 0x3FB        // BCD 1s, 10s
#define MINIMUM_HUMIDITY_INDOOR             0x3FD        // BCD 1s, 10s
#define MAXIMUM_HUMIDITY_INDOOR             0x3FF        // BCD 1s, 10s
#define TIME_MINIMUM_HUMIDITY_INDOOR        0x401        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MINIMUM_HUMIDITY_INDOOR        0x405        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define TIME_MAXIMUM_HUMIDITY_INDOOR        0x40B        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_HUMIDITY_INDOOR        0x40F        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define LOW_ALARM_HUMIDITY_INDOOR           0x415        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define HIGH_ALARM_HUMIDITY_INDOOR          0x417        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define REL_HUMIDITY_OUTDOOR                0x419        // BCD 1s, 10s
#define MINIMUM_HUMIDITY_OUTDOOR            0x41B        // BCD 1s, 10s
#define MAXIMUM_HUMIDITY_OUTDOOR            0x41D        // BCD 1s, 10s
#define TIME_MINIMUM_HUMIDITY_OUTDOOR       0x41F        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MINIMUM_HUMIDITY_OUTDOOR       0x423        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define TIME_MAXIMUM_HUMIDITY_OUTDOOR       0x429        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_HUMIDITY_OUTDOOR       0x42D        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define LOW_ALARM_HUMIDITY_OUTDOOR          0x433        // BCD offset 30 0.01s, 0.1s, 1s, 10s
#define HIGH_ALARM_HUMIDITY_OUTDOOR         0x435        // BCD offset 30 0.01s, 0.1s, 1s, 10s

#define LAST_HOUR_RAINCOUNT                 0x446        //
#define LAST_MINUTES_RAINCOUNT              0x479        //
#define RAIN_24_HOURS                       0x497        // BCD 0.01s, 0.1s, 1s, 10s, 100s, 1000s
#define RAIN_MAXIMUM_24_HOURS               0x49D        // BCD 0.01s, 0.1s, 1s, 10s, 100s, 1000s
#define TIME_MAXIMUM_24_HOURS               0x4A3        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_24_HOURS               0x4A7        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define ALARM_RAIN_24_HOURS                 0x4AE        // BCD 0.01s, 0.1s, 1s, 10s, 100s, 1000s
#define RAIN_1_HOUR                         0x4B4        // BCD 0.01s, 0.1s, 1s, 10s, 100s, 1000s
#define RAIN_MAXIMUM_1_HOUR                 0x4BA        // BCD 0.01s, 0.1s, 1s, 10s, 100s, 1000s
#define TIME_MAXIMUM_1_HOUR                 0x4C0        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_1_HOUR                 0x4C4        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define ALARM_RAIN_1_HOUR                   0x4CB        // BCD 0.01s, 0.1s, 1s, 10s, 100s, 1000s
#define TOTAL_RAIN                          0x4D2        // BCD 0.01s, 0.1s, 1s, 10s, 100s, 1000s
#define TIME_TOTAL_RAIN                     0x4D8        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_TOTAL_RAIN                     0x4DC        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s

#define WIND_MINIMUM                        0x4EE        //
#define WIND_MAXIMUM                        0x4F4        //
#define TIME_MINIMUM_WIND                   0x4F8        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MINIMUM_WIND                   0x4FC        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define TIME_MAXIMUM_WIND                   0x502        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_WIND                   0x506        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define LOW_WIND_ALARM_SETTING              0x50E        // BCD 1s, 10s, 100s, 1000 Speed km/h x 100
#define HIGH_WIND_ALARM_SETTING             0x514        // BCD 1s, 10s, 100s, 1000 Speed km/h x 100
#define WIND_DIRECTION_ALARM_SETTING        0x518        // n x 22.5
#define WIND_SPEED_AND_DIRECTION            0x527        //

#define ABSOLUTE_AIR_PRESSURE_HPA           0x5D8        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s
#define ABSOLUTE_AIR_PRESSURE_HG            0x5DD        // BCD Hg 0.01s, 0.1s, 1s, 10s, 100s
#define RELATIVE_AIR_PRESSURE_HPA           0x5E2        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s
#define RELATIVE_AIR_PRESSURE_HG            0x5E7        // BCD Hg 0.01s, 0.1s, 1s, 10s, 100s
#define AIR_PRESSURE_CORRECTION_HPA         0x5EC        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s
#define AIR_PRESSURE_CORRECTION_HG          0x5F1        // BCD Hg 0.01s, 0.1s, 1s, 10s, 100s
#define ABSOLUTE_AIR_PRESSURE_MINIMUM_HPA   0x5F6        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s
#define ABSOLUTE_AIR_PRESSURE_MINIMUM_HG    0x5FB        // BCD Hg 0.01s, 0.1s, 1s, 10s, 100s
#define RELATIVE_AIR_PRESSURE_MINIMUM_HPA   0x600        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s
#define RELATIVE_AIR_PRESSURE_MINIMUM_HG    0x605        // BCD Hg 0.01s, 0.1s, 1s, 10s, 100s
#define ABSOLUTE_AIR_PRESSURE_MAXIMUM_HPA   0x60A        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s
#define ABSOLUTE_AIR_PRESSURE_MAXIMUM_HG    0x60F        // BCD Hg 0.01s, 0.1s, 1s, 10s, 100s
#define RELATIVE_AIR_PRESSURE_MAXIMUM_HPA   0x614        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s
#define RELATIVE_AIR_PRESSURE_MAXIMUM_HG    0x619        // BCD Hg 0.01s, 0.1s, 1s, 10s, 100s
#define TIME_MINIMUM_AIR_PRESSURE           0x61E        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MINIMUM_AIR_PRESSURE           0x622        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define TIME_MAXIMUM_AIR_PRESSURE           0x628        // BCD mn 1s, 10s, hours 1s, 10s
#define DATE_MAXIMUM_AIR_PRESSURE           0x62C        // BCD day 1s, 10s, mounth 1s, 10s, year 1s, 10s
#define LOW_ALARM_AIR_PRESSURE_HPA          0x63C        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s
#define HIGH_ALARM_AIR_PRESSURE_HPA         0x650        // BCD hPa 0.1s, 1s, 10s, 100s, 1000s

#define HISTORY_SAVING_INTERVAL             0x6B2        // minutes
#define COUNTDOWN_TO_NEXT_SAVING            0x6B5        // minutes
#define TIME_LAST_RECORD_MINUTES            0x6B8        // 1, 10
#define TIME_LAST_RECORD_HOURS              0x6BA        // 1, 10
#define DATE_LAST_RECORD                    0x6BC        // BCD day 1, 10, mounth 1, 10, year 1, 10
#define POINTER_TO_LAST_WRITTEN_RECORD      0x6C2
#define NUMBER_OF_RECORDS                   0x6C4


// define the station configuration structure
typedef struct
{
    double          inTemperatureMinConf;
    bool            inTemperatureMinAlarmConf;
    double          inTemperatureMaxConf;
    bool            inTemperatureMaxAlarmConf;
    double          outTemperatureMinConf;
    bool            outTemperatureMinAlarmConf;
    double          outTemperatureMaxConf;
    bool            outTemperatureMaxAlarmConf;
    double          windChillTemperatureMinConf;
    bool            windChillTemperatureMinAlarmConf;
    double          windChillTemperatureMaxConf;
    bool            windChillTemperatureMaxAlarmConf;
    double          dewPointTemperatureMinConf;
    bool            dewPointTemperatureMinAlarmConf;
    double          dewPointTemperatureMaxConf;
    bool            dewPointTemperatureMaxAlarmConf;

    double          rain1HConf;
    bool            rain1HAlarmConf;
    double          rain1DayConf;
    bool            rain1DayAlarmConf;

    double          inHumidityMinConf;
    bool            inHumidityMinAlarmConf;
    double          inHumidityMaxConf;
    bool            inHumidityMaxAlarmConf;

    double          outHumidityMinConf;
    bool            outHumidityMinAlarmConf;
    double          outHumidityMaxConf;
    bool            outHumidityMaxAlarmConf;

    double          pressureMinConf;
    bool            pressureMinAlarmConf;
    double          pressureMaxConf;
    bool            pressureMaxAlarmConf;

    double          windSpeedMinConf;
    bool            windSpeedMinAlarmConf;
    double          windSpeedMaxConf;
    bool            windSpeedMaxAlarmConf;

    double          windDirectionConf;
    bool            windDirectionAlarmConf;

} ws2355ConfigurationType;



class Ws2355
{
public:

    Ws2355( Configuration * );
    ~Ws2355();

    int                     fdStation;

    // Configuration accessors
    double                  getInTemperatureMinConf( void ) { return theStationConfiguration.inTemperatureMinConf; }
    bool                    getInTemperatureMinAlarmConf( void ) { return theStationConfiguration.inTemperatureMinAlarmConf; }
    double                  getInTemperatureMaxConf( void ) { return theStationConfiguration.inTemperatureMaxConf; }
    bool                    getInTemperatureMaxAlarmConf( void ) { return theStationConfiguration.inTemperatureMaxAlarmConf; }
    double                  getOutTemperatureMinConf( void ) { return theStationConfiguration.outTemperatureMinConf; }
    bool                    getOutTemperatureMinAlarmConf( void ) { return theStationConfiguration.outTemperatureMinAlarmConf; }
    double                  getOutTemperatureMaxConf( void ) { return theStationConfiguration.outTemperatureMaxConf; }
    bool                    getOutTemperatureMaxAlarmConf( void ) { return theStationConfiguration.outTemperatureMaxAlarmConf; }
    double                  getWindChillTemperatureMinConf( void ) { return theStationConfiguration.windChillTemperatureMinConf; }
    bool                    getWindChillTemperatureMinAlarmConf( void ) { return theStationConfiguration.windChillTemperatureMinAlarmConf; }
    double                  getWindChillTemperatureMaxConf( void ) { return theStationConfiguration.windChillTemperatureMaxConf; }
    bool                    getWindChillTemperatureMaxAlarmConf( void ) { return theStationConfiguration.windChillTemperatureMaxAlarmConf; }
    double                  getDewPointTemperatureMinConf( void ) { return theStationConfiguration.dewPointTemperatureMinConf; }
    bool                    getDewPointTemperatureMinAlarmConf( void ) { return theStationConfiguration.dewPointTemperatureMinAlarmConf; }
    double                  getDewPointTemperatureMaxConf( void ) { return theStationConfiguration.dewPointTemperatureMaxConf; }
    bool                    getDewPointTemperatureMaxAlarmConf( void ) { return theStationConfiguration.dewPointTemperatureMaxAlarmConf; }

    double                  getRain1HConf( void ) { return theStationConfiguration.rain1HConf; }
    bool                    getRain1HAlarmConf( void ) { return theStationConfiguration.rain1HAlarmConf; }
    double                  getRain1DayConf( void ) { return theStationConfiguration.rain1DayConf; }
    bool                    getRain1DayAlarmConf( void ) { return theStationConfiguration.rain1DayAlarmConf; }

    double                  getInHumidityMinConf( void ) { return theStationConfiguration.inHumidityMinConf; }
    bool                    getInHumidityMinAlarmConf( void ) { return theStationConfiguration.inHumidityMinAlarmConf; }
    double                  getInHumidityMaxConf( void ) { return theStationConfiguration.inHumidityMaxConf; }
    bool                    getInHumidityMaxAlarmConf( void ) { return theStationConfiguration.inHumidityMaxAlarmConf; }

    double                  getOutHumidityMinConf( void ) { return theStationConfiguration.outHumidityMinConf; }
    bool                    getOutHumidityMinAlarmConf( void ) { return theStationConfiguration.outHumidityMinAlarmConf; }
    double                  getOutHumidityMaxConf( void ) { return theStationConfiguration.outHumidityMaxConf; }
    bool                    getOutHumidityMaxAlarmConf( void ) { return theStationConfiguration.outHumidityMaxAlarmConf; }

    double                  getPressureMinConf( void ) { return theStationConfiguration.pressureMinConf; }
    bool                    getPressureMinAlarmConf( void ) { return theStationConfiguration.pressureMinAlarmConf; }
    double                  getPressureMaxConf( void ) { return theStationConfiguration.pressureMaxConf; }
    bool                    getPressureMaxAlarmConf( void ) { return theStationConfiguration.pressureMaxAlarmConf; }

    double                  getWindSpeedMinConf( void ) { return theStationConfiguration.windSpeedMinConf; }
    bool                    getWindSpeedMinAlarmConf( void ) { return theStationConfiguration.windSpeedMinAlarmConf; }
    double                  getWindSpeedMaxConf( void ) { return theStationConfiguration.windSpeedMaxConf; }
    bool                    getWindSpeedMaxAlarmConf( void ) { return theStationConfiguration.windSpeedMaxAlarmConf; }

    double                  getWindDirectionConf( void ) { return theStationConfiguration.windDirectionConf; }
    bool                    getWindDirectionAlarmConf( void ) { return theStationConfiguration.windDirectionAlarmConf; }

    void                    setInTemperatureMinConf( double val ) { theStationConfiguration.inTemperatureMinConf = val; }
    void                    setInTemperatureMinAlarmConf( bool val ) { theStationConfiguration.inTemperatureMinAlarmConf = val; }
    void                    setInTemperatureMaxConf( double val ) { theStationConfiguration.inTemperatureMaxConf = val; }
    void                    setInTemperatureMaxAlarmConf( bool val ) { theStationConfiguration.inTemperatureMaxAlarmConf = val; }
    void                    setOutTemperatureMinConf( double val ) { theStationConfiguration.outTemperatureMinConf = val; }
    void                    setOutTemperatureMinAlarmConf( bool val ) { theStationConfiguration.outTemperatureMinAlarmConf = val; }
    void                    setOutTemperatureMaxConf( double val ) { theStationConfiguration.outTemperatureMaxConf = val; }
    void                    setOutTemperatureMaxAlarmConf( bool val ) { theStationConfiguration.outTemperatureMaxAlarmConf = val; }
    void                    setWindChillTemperatureMinConf( double val ) { theStationConfiguration.windChillTemperatureMinConf = val; }
    void                    setWindChillTemperatureMinAlarmConf( bool val ) { theStationConfiguration.windChillTemperatureMinAlarmConf = val; }
    void                    setWindChillTemperatureMaxConf( double val ) { theStationConfiguration.windChillTemperatureMaxConf = val; }
    void                    setWindChillTemperatureMaxAlarmConf( bool val ) { theStationConfiguration.windChillTemperatureMaxAlarmConf = val; }
    void                    setDewPointTemperatureMinConf( double val ) { theStationConfiguration.dewPointTemperatureMinConf = val; }
    void                    setDewPointTemperatureMinAlarmConf( bool val ) { theStationConfiguration.dewPointTemperatureMinAlarmConf = val; }
    void                    setDewPointTemperatureMaxConf( double val ) { theStationConfiguration.dewPointTemperatureMaxConf = val; }
    void                    setDewPointTemperatureMaxAlarmConf( bool val ) { theStationConfiguration.dewPointTemperatureMaxAlarmConf = val; }
    void                    writeDewPointMaxAlarm( bool val );
    void                    setRain1HConf( double val ) { theStationConfiguration.rain1HConf = val; }
    void                    setRain1HAlarmConf( bool val ) { theStationConfiguration.rain1HAlarmConf = val; }
    void                    setRain1DayConf( double val ) { theStationConfiguration.rain1DayConf = val; }
    void                    setRain1DayAlarmConf( bool val ) { theStationConfiguration.rain1DayAlarmConf = val; }
    void                    setInHumidityMinConf( double val ) { theStationConfiguration.inHumidityMinConf = val; }
    void                    setInHumidityMinAlarmConf( bool val ) { theStationConfiguration.inHumidityMinAlarmConf = val; }
    void                    setInHumidityMaxConf( double val ) { theStationConfiguration.inHumidityMaxConf = val; }
    void                    setInHumidityMaxAlarmConf( bool val ) { theStationConfiguration.inHumidityMaxAlarmConf = val; }
    void                    setOutHumidityMinConf( double val ) { theStationConfiguration.outHumidityMinConf = val; }
    void                    setOutHumidityMinAlarmConf( bool val ) { theStationConfiguration.outHumidityMinAlarmConf = val; }
    void                    setOutHumidityMaxConf( double val ) { theStationConfiguration.outHumidityMaxConf = val; }
    void                    setOutHumidityMaxAlarmConf( bool val ) { theStationConfiguration.outHumidityMaxAlarmConf = val; }

    void                    setPressureMinConf( double val ) { theStationConfiguration.pressureMinConf = val; }
    void                    setPressureMinAlarmConf( bool val ) { theStationConfiguration.pressureMinAlarmConf = val; }
    void                    setPressureMaxConf( double val ) { theStationConfiguration.pressureMaxConf = val; }
    void                    setPressureMaxAlarmConf( bool val ) { theStationConfiguration.pressureMaxAlarmConf = val; }

    void                    setWindSpeedMinConf( double val ) { theStationConfiguration.windSpeedMinConf = val; }
    void                    setWindSpeedMinAlarmConf( bool val ) { theStationConfiguration.windSpeedMinAlarmConf = val; }
    void                    setWindSpeedMaxConf( double val ) { theStationConfiguration.windSpeedMaxConf = val; }
    void                    setWindSpeedMaxAlarmConf( bool val ) { theStationConfiguration.windSpeedMaxAlarmConf = val; }

    void                    setWindDirectionConf( double val ) { theStationConfiguration.windDirectionConf = val; }
    void                    setWindDirectionAlarmConf( bool val ) { theStationConfiguration.windDirectionAlarmConf = val; }

    CurrentData             *getTheCurrentData( void ) { return theCurrentData; }
    Connection              *getTheConnection( void ) { return theConnection; }
    DataLogged              *getTheDataLogged( void ) { return theDataLogged; }
    void                    dataReading( void );
    void                    readCurrentValues( void );

    double                  readTemperatureIndoor( int );
    double                  readTemperatureOutdoor( int );
    int                     readHumidityIndoor( void );
    int                     readHumidityOutdoor( void );
    double                  readDewpoint( int );

    double                  readWindchill( int );
    double                  readRelPressure( double );
    double                  readAbsPressure( double );
    double                  readWindCurrent( double , double * );
    double                  readRain1h( double );
    double                  readRain24h( double );
    double                  readRainTotal( double );
    void                    readTendencyForecast( unsigned char *, unsigned char * );

    // Alarms
    bool                    readPressureMaxAlarm( void );
    double                  readPressureMax( double );
    bool                    readPressureMinAlarm( void );
    double                  readPressureMin( double );
    bool                    readOutdoorTemperatureMaxAlarm( void );
    double                  readOutdoorTemperatureMax( int );
    bool                    readOutdoorTemperatureMinAlarm( void );
    double                  readOutdoorTemperatureMin( int );
    bool                    readIndoorTemperatureMaxAlarm( void );
    double                  readIndoorTemperatureMax( int );
    bool                    readIndoorTemperatureMinAlarm( void );
    double                  readIndoorTemperatureMin( int );
    bool                    readDewpointMaxAlarm( void );
    double                  readDewpointMax( int );
    bool                    readDewpointMinAlarm( void );
    double                  readDewpointMin( int );
    bool                    readWindchillMaxAlarm( void );
    double                  readWindchillMax( int );
    bool                    readWindchillMinAlarm( void );
    double                  readWindchillMin( int );
    bool                    readOutdoorHumidityMaxAlarm( void );
    int                     readOutdoorHumidityMax( void );
    bool                    readOutdoorHumidityMinAlarm( void );
    int                     readOutdoorHumidityMin( void );
    bool                    readIndoorHumidityMaxAlarm( void );
    int                     readIndoorHumidityMax( void );
    bool                    readIndoorHumidityMinAlarm( void );
    int                     readIndoorHumidityMin( void );
    bool                    readRain1hAlarm( void );
    double                  readRain1hMax( double );
    bool                    readRain24hAlarm( void );
    double                  readRain24hMax( double );
    bool                    readWindDirectionAlarm( void );
    double                  readWindDirection( void );
    bool                    readWindSpeedMaxAlarm( void );
    double                  readWindSpeedMax( double );
    bool                    readWindSpeedMinAlarm( void );
    double                  readWindSpeedMin( double );

private:
    ws2355ConfigurationType theStationConfiguration;
    Configuration           *theGlobalConfiguration;
    CurrentData             *theCurrentData;
    Connection              *theConnection;
    DataLogged              *theDataLogged;

    Ws2355Thread            *newThread;


    void                    writeSettings( void );
    void                    readSettings( void );

    void                    temperatureIndoorMinMax( int, double *, double *,QDateTime *, QDateTime * );
    int                     temperatureIndoorReset( char );
    void                    temperatureOutdoorMinMax( int, double *, double *, QDateTime *, QDateTime * );
    int                     temperatureOutdoorReset( char );
    void                    dewpointMinMax( int, double *, double *, QDateTime *, QDateTime * );
    int                     dewpointReset( char );
    int                     humidityIndoorAll( int *, int *, QDateTime *, QDateTime * );
    int                     humidityIndoorReset( char );
    int                     humidityOutdoorAll( int *, int *, QDateTime *, QDateTime * );
    int                     humidityOutdoorReset( char );
    double                  windAll( double , int *, double *);
    double                  windMinMax( double, double *, double *, QDateTime *, QDateTime * );
    int                     windReset( char );
    void                    windchillMinMax( int, double *, double *, QDateTime *, QDateTime *);
    int                     windchillReset( char );
    double                  rain1hAll( double , double *, QDateTime * );
    int                     rain1hMaxReset( void );
    int                     rain1hReset( void );
    double                  rain24hAll( double, double *, QDateTime * );
    int                     rain24hMaxReset( void );
    int                     rain24hReset( void );
    double                  rainTotalAll( double , QDateTime *);
    int                     rainTotalReset( void );
    void                    relPressureMinMax( double , double *, double *, QDateTime *, QDateTime *);
    void                    absPressureMinMax( double, double *, double *, QDateTime *, QDateTime * );
    int                     pressureReset( char );
    double                  pressureCorrection( double );
    int                     readHistoryInfo( int *, int *, QDateTime *, int * );
    void                    light( int );
    void                    readErrorExit( void );
    void                    writeErrorExit( void );
    void                    addressEncoder( int , unsigned char * );
    void                    dataEncoder( int, unsigned char, unsigned char *, unsigned char * );
    unsigned char           numberOfEncoder( int );
    unsigned char           commandCheck0123( unsigned char *, int );
    unsigned char           commandCheck4( int );
    unsigned char           dataChecksum( unsigned char *, int );
    int                     initialize( void );
    int                     readData( int, int, unsigned char *, unsigned char * );
    int                     writeData( int, int, unsigned char, unsigned char *, unsigned char *);
    int                     readSafe( int, int, unsigned char *, unsigned char *);
    int                     writeSafe( int, int, unsigned char , unsigned char *, unsigned char *);
    int                     readDevice( unsigned char *, int );
    int                     writeDevice( unsigned char *, int );
    void                    reset06( void );

    int                     readHistoryRecord( int, double *, double *, double *, int *, int *, double *, double *, double *, double *, double * );


};

#endif // WS2355_H
