/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "YearlyDataTab.h"

YearlyDataTab::YearlyDataTab( QWidget *parent, Ws2355 *station, Configuration *config ) :
    QFrame(parent)
{
    theStation = station;
    theGlobalConfiguration = config;

    createTab();
}

/**
 * Destructor
 */
YearlyDataTab::~YearlyDataTab()
{
    delete yearlyTemperaturesGraph;
    delete yearlyRainGraph;
    delete yearlyHumidityGraph;
    delete yearlyPressureGraph;
    delete yearlyWindGraph;
    delete year;

    delete temperaturesGroup;
    delete rainGroup;
    delete humidityGroup;
    delete pressureGroup;
    delete windGroup;
}

/**
 * When the language change, update all text fields.
 * Capture QEvent::LanguageChange and forward all other events
 *
 * @param event
 */
void YearlyDataTab::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange)
    {
        // Set all texts
        temperaturesGroup->setTitle( trUtf8( "Températures" ));
        rainGroup->setTitle( trUtf8( "Précipitations" ));
        humidityGroup->setTitle( trUtf8( "Humidité" ));
        pressureGroup->setTitle( trUtf8( "Pression" ));
        windGroup->setTitle( trUtf8( "Vent" ));

        yearlyTemperaturesGraph->setText();
        yearlyRainGraph->setText();
        yearlyHumidityGraph->setText();
        yearlyPressureGraph->setText();
        yearlyWindGraph->setText();
    }
    else
    {
        QWidget::changeEvent(event);
    }
}

/**
 * Create the tab
 */
void YearlyDataTab::createTab( void )
{
    // Temperatures
    temperaturesGroup = new QGroupBox( trUtf8( "Températures" ));
    yearlyTemperaturesGraph =  new TemperaturesGraph( 1, 12, 1, theGlobalConfiguration->getTemperatureUnit() );

    // Rain
    rainGroup = new QGroupBox( trUtf8( "Précipitations" ));
    yearlyRainGraph =  new RainGraph(0, 12, 1 );

    // Humidity
    humidityGroup = new QGroupBox( trUtf8( "Humidité" ));
    yearlyHumidityGraph =  new HumidityGraph( 1, 12, 1 );

    // Pressure
    pressureGroup = new QGroupBox( trUtf8( "Pression" ));
    yearlyPressureGraph =  new PressureGraph( 1, 12, 1 );

    // Wind
    windGroup = new QGroupBox( trUtf8( "Vent" ));
    yearlyWindGraph =  new WindGraph( 1, 12, 1 );

    // Year
    QGroupBox *yearGroup = new QGroupBox( );

    year = new QDateEdit( );
    year->setDisplayFormat( "yyyy" );
    year->setMaximumSize( 200,35 );
    year->setDate( QDate::currentDate() );
    connect( year, SIGNAL( dateChanged( QDate ) ), this, SLOT( yearChanged( QDate ) ));

    // Set up layout

    QGridLayout *temperaturesLayout = new QGridLayout;
    temperaturesLayout->addWidget( yearlyTemperaturesGraph->getPlot(), 0, 0 );
    temperaturesGroup->setLayout( temperaturesLayout );

    QGridLayout *rainLayout = new QGridLayout;
    rainLayout->addWidget( yearlyRainGraph->getPlot(), 0, 0 );
    rainGroup->setLayout( rainLayout );

    QGridLayout *humidityLayout = new QGridLayout;
    humidityLayout->addWidget( yearlyHumidityGraph->getPlot(), 0, 0 );
    humidityGroup->setLayout( humidityLayout );

    QGridLayout *pressureLayout = new QGridLayout;
    pressureLayout->addWidget( yearlyPressureGraph->getPlot(), 0, 0 );
    pressureGroup->setLayout( pressureLayout );

    QGridLayout *windLayout = new QGridLayout;
    windLayout->addWidget( yearlyWindGraph->getPlot(), 0, 0 );
    windGroup->setLayout( windLayout );

    QGridLayout *yearLayout = new QGridLayout;
    yearLayout->addWidget( year, 0, 0 );
    yearGroup->setLayout( yearLayout );

    QGridLayout *currentLayout = new QGridLayout();
    currentLayout->addWidget( temperaturesGroup, 0, 0 );
    currentLayout->addWidget( rainGroup, 0, 1 );
    currentLayout->addWidget( humidityGroup, 0, 2 );
    currentLayout->addWidget( pressureGroup, 1, 0 );
    currentLayout->addWidget( windGroup, 1, 1 );
    currentLayout->addWidget( yearGroup, 1, 2 );

    this->setLayout( currentLayout );

    updateGraphics( year->date() );

}

/**
 * SLOT : Change displayed data when a new year is selected
 */
void YearlyDataTab::yearChanged(const QDate & date)
{
    yearlyTemperaturesGraph->setXAxisMax( 12 );
    yearlyRainGraph->setXAxisMax( 12 );
    yearlyHumidityGraph->setXAxisMax( 12 );
    yearlyPressureGraph->setXAxisMax( 12 );
    yearlyWindGraph->setXAxisMax( 12 );

    updateGraphics( year->date() );

}

void YearlyDataTab::updateGraphics( QDate date  )
{
    theStation->getTheDataLogged()->readData( date, BY_YEAR );

    yearlyTemperaturesGraph->update( theStation->getTheDataLogged()->getData()->inTemperatureX,
                                      theStation->getTheDataLogged()->getData()->inTemperatureY,
                                      theStation->getTheDataLogged()->getData()->outTemperatureX,
                                      theStation->getTheDataLogged()->getData()->outTemperatureY,
                                      theStation->getTheDataLogged()->getData()->dewPointTemperatureX,
                                      theStation->getTheDataLogged()->getData()->dewPointTemperatureY );

    yearlyHumidityGraph->update( theStation->getTheDataLogged()->getData()->inHumidityX,
                                  theStation->getTheDataLogged()->getData()->inHumidityY,
                                  theStation->getTheDataLogged()->getData()->outHumidityX,
                                  theStation->getTheDataLogged()->getData()->outHumidityY );

    yearlyPressureGraph->update( theStation->getTheDataLogged()->getData()->pressureX,
                                  theStation->getTheDataLogged()->getData()->pressureY );

    yearlyRainGraph->update( theStation->getTheDataLogged()->getData()->rainX,
                              theStation->getTheDataLogged()->getData()->rainY);

    yearlyWindGraph->update( theStation->getTheDataLogged()->getData()->windDirectionX,
                              theStation->getTheDataLogged()->getData()->windDirectionY,
                              theStation->getTheDataLogged()->getData()->windSpeedX,
                              theStation->getTheDataLogged()->getData()->windSpeedY );
}


